#include "gui/gui.hpp"
#include "imgui.h"
#include "imgui_stdlib.h"
#include "imgui_impl_glfw.h"
#include "imgui_impl_opengl3.h"
#include "gui/window.hpp"
#include "gui/gui_data.hpp"
#include "character_sheet.hpp"
#include "controller.hpp"

gui::Gui::Gui(GuiData &data, menu::MenuData &menuData) : data(data), menu(menuData), font("font/DejaVuSans.ttf") {
	SPDLOG_DEBUG("Creating GUI");
	(void) ImGui::CreateContext();
	ImGui::StyleColorsDark();
	(void) ImGui::GetIO().Fonts->AddFontFromFileTTF(font.c_str(), 18.0f);
	(void) ImGui_ImplGlfw_InitForOpenGL(data.get_window().get_window().get(), true);
	initialized = ImGui_ImplOpenGL3_Init("#version 130");
	SPDLOG_DEBUG("Initialized: {}", initialized);
}

gui::Gui::~Gui() noexcept {
	ImGui_ImplOpenGL3_Shutdown();
	ImGui_ImplGlfw_Shutdown();
	ImGui::DestroyContext();
}

void gui::Gui::render_gui(const Controller &controller) {
	if (initialized) {
		constexpr ImGuiWindowFlags flags = ImGuiWindowFlags_MenuBar |
										   ImGuiWindowFlags_NoMove |
										   ImGuiWindowFlags_NoResize |
										   ImGuiWindowFlags_NoCollapse |
										   ImGuiWindowFlags_NoTitleBar;

		ImGui_ImplOpenGL3_NewFrame();
		ImGui_ImplGlfw_NewFrame();
		ImGui::NewFrame();

		//TODO GUI
		{
			const ImGuiViewport &viewport = *ImGui::GetMainViewport();
			ImGui::SetNextWindowPos(viewport.Pos);
			ImGui::SetNextWindowSize(viewport.Size);
		}

		// Never collapsed.
		(void) ImGui::Begin("Billy", nullptr, flags);
		menu.gui();

		{
			ImGui::BeginChild("carac", ImVec2(0, ImGui::GetWindowHeight() * 0.2f), true);
			ImGui::Text("Caractère");
			// Remove label
			ImGui::PushItemWidth(-1);
			ImGui::InputTextMultiline("Caractère", &data.billy.caractere);
			ImGui::PopItemWidth();
			ImGui::EndChild();
		}

		ImGui::BeginGroup();
		{
			ImGui::BeginChild("habilete", ImVec2(ImGui::GetWindowWidth() / 3, ImGui::GetWindowHeight() * 0.2f), true);
			ImGui::Text("Habileté");
			ImGui::EndChild();
		}
		ImGui::SameLine();

		{
			ImGui::BeginChild("adresse", ImVec2(ImGui::GetWindowWidth() / 3, ImGui::GetWindowHeight() * 0.2f), true);
			ImGui::Text("Adresse");
			ImGui::Text("Bottom pane1");
			ImGui::Text("Bottom pane2");
			ImGui::Text("Bottom pane3");
			ImGui::Text("Bottom pane4");
			ImGui::EndChild();
		}

		{
			ImGui::BeginChild("endurance", ImVec2(ImGui::GetWindowWidth() / 3, ImGui::GetWindowHeight() * 0.2f), true);
			ImGui::Text("Endurance");
			ImGui::Text("Bottom pane1");
			ImGui::Text("Bottom pane2");
			ImGui::Text("Bottom pane3");
			ImGui::Text("Bottom pane4");
			ImGui::EndChild();
		}
		ImGui::SameLine();

		{
			ImGui::BeginChild("chance", ImVec2(ImGui::GetWindowWidth() / 3, ImGui::GetWindowHeight() * 0.2f), true);
			ImGui::Text("Chance");
			ImGui::Text("Bottom pane1");
			ImGui::Text("Bottom pane2");
			ImGui::Text("Bottom pane3");
			ImGui::Text("Bottom pane4");
			ImGui::EndChild();
		}
		ImGui::EndGroup();
		ImGui::SameLine();
		{
			ImGui::BeginChild("stats secondaire", ImVec2(ImGui::GetWindowWidth() / 3, ImGui::GetWindowHeight() * 0.4f), true);
			ImGui::Text("STAT. SECONDAIRES");
			ImGui::EndChild();
		}

		ImGui::End();

		controller.control_menu();

		ImGui::Render();
	}
}

void gui::Gui::render_gpu() const {
	if (initialized) {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
		ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
	}
}
